<?php

namespace SmashPig\Core\DataStores;

use PHPQueue\Interfaces\FifoQueueStore;
use SmashPig\Core\Context;
use SmashPig\CrmLink\Messages\SourceFields;

class QueueWrapper {

	/**
	 * @param string $queueName
	 * @param array|JsonSerializableObject $message
	 * @throws \SmashPig\Core\ConfigurationKeyException
	 * @throws \Exception
	 */
	public static function push( string $queueName, $message ) {
		if ( $message instanceof JsonSerializableObject ) {
			$message = json_decode( $message->toJson(), true );
		}
		$queue = self::getQueue( $queueName );
		SourceFields::addToMessage( $message );
		$queue->push( $message );
	}

	/**
	 * @param string $queueName
	 * @return FifoQueueStore
	 * @throws \SmashPig\Core\ConfigurationKeyException
	 */
	public static function getQueue( string $queueName ): FifoQueueStore {
		$config = Context::get()->getGlobalConfiguration();
		$key = "data-store/$queueName";

		// Examine the config node for a queue name
		$node = $config->val( $key );
		if (
			empty( $node['constructor-parameters'] ) ||
			empty( $node['constructor-parameters'][0]['queue'] )
		) {
			$nameParam = [
				'data-store' => [
					$queueName => [
						'constructor-parameters' => [
							[
								'queue' => $queueName
							]
						]
					]
				]
			];
			$config->override( $nameParam );
		}

		return $config->object( $key );
	}

}
